@@ -24,7 +24,7 @@ import bzrlib
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
-from bzrlib import help
+from bzrlib import help, urlutils
 import shelf
 """)
 
@@ -37,9 +37,12 @@ import os.path
 
 import bzrlib.builtins
 import bzrlib.commands
+from bzrlib.branch import Branch
+from bzrlib.bzrdir import BzrDir
 from bzrlib.commands import get_cmd_object
 from bzrlib.errors import BzrCommandError
 import bzrlib.ignores
+from bzrlib.trace import note
 from bzrlib.option import Option
 sys.path.insert(0, os.path.realpath(os.path.join(os.path.dirname(__file__),
                                                  "external")))
@@ -406,11 +409,13 @@ class cmd_zap(BzrToolsCommand):
     the branch has no new commits (relative to its parent).
     """
     takes_options = [Option("branch", help="Remove associated branch from"
-                                           " repository.")]
+                                           " repository."),
+                     Option('force', help='Delete tree even if contents are'
+                     ' modified.')]
     takes_args = ["checkout"]
-    def run(self, checkout, branch=False):
+    def run(self, checkout, branch=False, force=False):
         from zap import zap
-        return zap(checkout, remove_branch=branch)
+        return zap(checkout, remove_branch=branch, allow_modified=force)
 
 
 class cmd_cbranch(BzrToolsCommand):
@@ -433,12 +438,16 @@ class cmd_cbranch(BzrToolsCommand):
     deprecated.
     """
     takes_options = [Option("lightweight",
-                            help="Create a lightweight checkout."), 'revision']
+                            help="Create a lightweight checkout."), 'revision',
+                     Option('files-from', type=unicode,
+                            help='Accelerate checkout using files from this'
+                                 ' tree.')]
     takes_args = ["source", "target?"]
-    def run(self, source, target=None, lightweight=False, revision=None):
+    def run(self, source, target=None, lightweight=False, revision=None,
+            files_from=None):
         from cbranch import cbranch
         return cbranch(source, target, lightweight=lightweight,
-                       revision=revision)
+                       revision=revision, files_from=files_from)
 
 
 class cmd_branches(BzrToolsCommand):
@@ -448,6 +457,16 @@ class cmd_branches(BzrToolsCommand):
         from branches import branches
         return branches(location)
 
+class cmd_trees(BzrToolsCommand):
+    """Scan a location for trees"""
+    takes_args = ['location?']
+    def run(self, location='.'):
+        from bzrlib.workingtree import WorkingTree
+        from bzrlib.transport import get_transport
+        t = get_transport(location)
+        for tree in WorkingTree.find_trees(location):
+            self.outf.write('%s\n' % t.relpath(
+                tree.bzrdir.root_transport.base))
 
 class cmd_multi_pull(BzrToolsCommand):
     """Pull all the branches under a location, e.g. a repository.
@@ -457,7 +476,6 @@ class cmd_multi_pull(BzrToolsCommand):
     """
     takes_args = ["location?"]
     def run(self, location=None):
-        from bzrlib.branch import Branch
         from bzrlib.transport import get_transport
         from bzrtools import iter_branch_tree
         if location is None:
@@ -491,23 +509,6 @@ class cmd_multi_pull(BzrToolsCommand):
                 print e
 
 
-class cmd_branch_mark(BzrToolsCommand):
-    """
-    Add, view or list branch markers <EXPERIMENTAL>
-
-    To add a mark, do 'bzr branch-mark MARK'.
-    To list marks, do 'bzr branch-mark' (this lists all marks for the branch's
-    repository).
-    To delete a mark, do 'bzr branch-mark --delete MARK'
-
-    These marks can be used to track a branch's status.
-    """
-    takes_args = ['mark?', 'branch?']
-    takes_options = [Option('delete', help='Delete this mark.')]
-    def run(self, mark=None, branch=None, delete=False):
-        from branch_mark import branch_mark
-        branch_mark(mark, branch, delete)
-
 
 class cmd_import(BzrToolsCommand):
     """Import sources from a directory, tarball or zip file
@@ -636,22 +637,12 @@ class cmd_rspush(BzrToolsCommand):
                       working_tree=not no_tree)
 
 
-class cmd_switch(BzrToolsCommand):
-    """Set the branch of a lightweight checkout and update."""
-
-    takes_args = ['to_location']
-
-    def run(self, to_location):
-        from switch import cmd_switch
-        cmd_switch().run(to_location)
-
-
 commands = [
             cmd_baz_import,
             cmd_baz_import_branch,
             cmd_branches,
+            cmd_trees,
             cmd_branch_history,
-            cmd_branch_mark,
             cmd_cbranch,
             cmd_cdiff,
             cmd_clean_tree,
@@ -664,7 +655,6 @@ commands = [
             cmd_shelf,
             cmd_shell,
             cmd_shelve,
-            cmd_switch,
             cmd_unshelve,
             cmd_zap,
             ]
@@ -683,6 +673,7 @@ def test_suite():
     import bzrtools
     import tests.clean_tree
     import tests.is_clean
+    import tests.test_rspush
     import tests.upstream_import
     import zap
     import tests.blackbox
@@ -701,4 +692,5 @@ def test_suite():
     result.addTest(tests.upstream_import.test_suite())
     result.addTest(zap.test_suite())
     result.addTest(TestLoader().loadTestsFromModule(tests.is_clean))
+    result.addTest(TestLoader().loadTestsFromModule(tests.test_rspush))
     return result
