@@ -24,11 +24,19 @@ from subprocess import Popen, PIPE
 import sys
 
 import bzrlib
-from bzrlib import urlutils
+from bzrlib import revision as _mod_revision, trace, urlutils
 import bzrlib.errors
-from bzrlib.errors import (BzrCommandError, NotBranchError, NoSuchFile,
-                           UnsupportedFormatError, TransportError,
-                           NoWorkingTree, PermissionDenied, ConnectionError)
+from bzrlib.errors import (
+    BzrCommandError,
+    BzrError,
+    ConnectionError,
+    NotBranchError,
+    NoSuchFile,
+    NoWorkingTree,
+    PermissionDenied,
+    UnsupportedFormatError,
+    TransportError,
+    )
 from bzrlib.bzrdir import BzrDir, BzrDirFormat
 from bzrlib.transport import get_transport
 
@@ -101,24 +109,9 @@ class NoRsync(Exception):
     def __init__(self, rsync_name):
         Exception.__init__(self, "%s not found." % rsync_name)
 
+
 def rsync(source, target, ssh=False, excludes=(), silent=False,
           rsync_name="rsync"):
-    """
-    >>> new_dir = tempfile.mkdtemp()
-    >>> old_dir = os.getcwd()
-    >>> os.chdir(new_dir)
-    >>> rsync("a", "b", silent=True)
-    Traceback (most recent call last):
-    RsyncNoFile: No such file...
-    >>> rsync(new_dir + "/a", new_dir + "/b", excludes=("*.py",), silent=True)
-    Traceback (most recent call last):
-    RsyncNoFile: No such file...
-    >>> rsync(new_dir + "/a", new_dir + "/b", excludes=("*.py",), silent=True, rsync_name="rsyncc")
-    Traceback (most recent call last):
-    NoRsync: rsyncc not found.
-    >>> os.chdir(old_dir)
-    >>> os.rmdir(new_dir)
-    """
     cmd = [rsync_name, "-av", "--delete"]
     if ssh:
         cmd.extend(('-e', 'ssh'))
@@ -188,6 +181,15 @@ def read_revision_history(fname):
     return [l.rstrip('\r\n') for l in
             codecs.open(fname, 'rb', 'utf-8').readlines()]
 
+
+def read_revision_info(path):
+    """Parse a last_revision file to determine revision_info"""
+    line = open(path, 'rb').readlines()[0].strip('\n')
+    revno, revision_id = line.split(' ', 1)
+    revno = int(revno)
+    return revno, revision_id
+
+
 class RsyncNoFile(Exception):
     def __init__(self, path):
         Exception.__init__(self, "No such file %s" % path)
@@ -196,12 +198,25 @@ class RsyncStreamIO(Exception):
     def __init__(self):
         Exception.__init__(self, "Error in rsync protocol data stream.")
 
-def get_revision_history(location):
+
+class NotStandalone(BzrError):
+
+    _format = '%(location) is not a standalone tree.'
+    _internal = False
+
+    def __init__(self, location):
+        BzrError.__init__(self, location=location)
+
+
+def get_revision_history(location, _rsync):
     tempdir = tempfile.mkdtemp('push')
+    my_rsync = _rsync
+    if my_rsync is None:
+        my_rsync = rsync
     try:
         history_fname = os.path.join(tempdir, 'revision-history')
         try:
-            cmd = rsync(location+'.bzr/revision-history', history_fname,
+            cmd = my_rsync(location+'.bzr/revision-history', history_fname,
                         silent=True)
         except RsyncNoFile:
             cmd = rsync(location+'.bzr/branch/revision-history', history_fname,
@@ -211,15 +226,39 @@ def get_revision_history(location):
         shutil.rmtree(tempdir)
     return history
 
-def history_subset(location, branch):
-    remote_history = get_revision_history(location)
+
+def get_revision_info(location, _rsync):
+    """Get the revsision_info for an rsync-able branch"""
+    tempdir = tempfile.mkdtemp('push')
+    my_rsync = _rsync
+    if my_rsync is None:
+        my_rsync = rsync
+    try:
+        info_fname = os.path.join(tempdir, 'last-revision')
+        cmd = rsync(location+'.bzr/branch/last-revision', info_fname,
+                    silent=True)
+        return read_revision_info(info_fname)
+    finally:
+        shutil.rmtree(tempdir)
+
+
+def history_subset(location, branch, _rsync=None):
     local_history = branch.revision_history()
-    if len(remote_history) > len(local_history):
-        return False
-    for local, remote in zip(remote_history, local_history):
-        if local != remote:
+    try:
+        remote_history = get_revision_history(location, _rsync)
+    except RsyncNoFile:
+        revno, revision_id = get_revision_info(location, _rsync)
+        if revision_id == _mod_revision.NULL_REVISION:
+            return True
+        return bool(revision_id.decode('utf-8') in local_history)
+    else:
+        if len(remote_history) > len(local_history):
             return False
-    return True
+        for local, remote in zip(remote_history, local_history):
+            if local != remote:
+                return False
+        return True
+
 
 def empty_or_absent(location):
     try:
@@ -228,59 +267,76 @@ def empty_or_absent(location):
     except RsyncNoFile:
         return True
 
-def rspush(tree, location=None, overwrite=False, working_tree=True):
-    push_location = get_push_data(tree)
-    if location is not None:
-        if not location.endswith('/'):
-            location += '/'
-        push_location = location
+def rspush(tree, location=None, overwrite=False, working_tree=True,
+    _rsync=None):
+    tree.lock_write()
+    try:
+        my_rsync = _rsync
+        if my_rsync is None:
+            my_rsync = rsync
+        if (tree.bzrdir.root_transport.base !=
+            tree.branch.bzrdir.root_transport.base):
+            raise NotStandalone(tree.bzrdir.root_transport.base)
+        if (tree.branch.get_bound_location() is not None):
+            raise NotStandalone(tree.bzrdir.root_transport.base)
+        if (tree.branch.repository.is_shared()):
+            raise NotStandalone(tree.bzrdir.root_transport.base)
+        push_location = get_push_data(tree)
+        if location is not None:
+            if not location.endswith('/'):
+                location += '/'
+            push_location = location
 
-    if push_location is None:
-        raise BzrCommandError("No rspush location known or specified.")
+        if push_location is None:
+            raise BzrCommandError("No rspush location known or specified.")
 
-    if (push_location.find('::') != -1):
-        usessh=False
-    else:
-        usessh=True
+        if (push_location.find('::') != -1):
+            usessh=False
+        else:
+            usessh=True
 
-    if (push_location.find('://') != -1 or
-        push_location.find(':') == -1):
-        raise BzrCommandError("Invalid rsync path %r." % push_location)
+        if (push_location.find('://') != -1 or
+            push_location.find(':') == -1):
+            raise BzrCommandError("Invalid rsync path %r." % push_location)
 
-    if working_tree:
-        clean, non_source = is_clean(tree)
-        if not clean:
-            print """Error: This tree has uncommitted changes or unknown (?) files.
-    Use "bzr status" to list them."""
-            sys.exit(1)
-        final_exclusions = non_source[:]
-    else:
-        wt = tree
-        final_exclusions = []
-        for path, status, kind, file_id, entry in wt.list_files():
-            final_exclusions.append(path)
+        if working_tree:
+            clean, non_source = is_clean(tree)
+            if not clean:
+                raise bzrlib.errors.BzrCommandError(
+                    'This tree has uncommitted changes or unknown'
+                    ' (?) files.  Use "bzr status" to list them.')
+                sys.exit(1)
+            final_exclusions = non_source[:]
+        else:
+            wt = tree
+            final_exclusions = []
+            for path, status, kind, file_id, entry in wt.list_files():
+                final_exclusions.append(path)
 
-    final_exclusions.extend(exclusions)
-    if not overwrite:
-        try:
-            if not history_subset(push_location, tree.branch):
-                raise bzrlib.errors.BzrCommandError("Local branch is not a"
-                                                    " newer version of remote"
-                                                    " branch.")
-        except RsyncNoFile:
-            if not empty_or_absent(push_location):
-                raise bzrlib.errors.BzrCommandError("Remote location is not a"
-                                                    " bzr branch (or empty"
-                                                    " directory)")
-        except RsyncStreamIO:
-            raise bzrlib.errors.BzrCommandError("Rsync could not use the"
-                " specified location.  Please ensure that"
-                ' "%s" is of the form "machine:/path".' % push_location)
-    print "Pushing to %s" % push_location
-    rsync(tree.basedir+'/', push_location, ssh=usessh,
-          excludes=final_exclusions)
+        final_exclusions.extend(exclusions)
+        if not overwrite:
+            try:
+                if not history_subset(push_location, tree.branch,
+                                      _rsync=my_rsync):
+                    raise bzrlib.errors.BzrCommandError(
+                        "Local branch is not a newer version of remote"
+                        " branch.")
+            except RsyncNoFile:
+                if not empty_or_absent(push_location):
+                    raise bzrlib.errors.BzrCommandError(
+                        "Remote location is not a bzr branch (or empty"
+                        " directory)")
+            except RsyncStreamIO:
+                raise bzrlib.errors.BzrCommandError("Rsync could not use the"
+                    " specified location.  Please ensure that"
+                    ' "%s" is of the form "machine:/path".' % push_location)
+        trace.note("Pushing to %s", push_location)
+        my_rsync(tree.basedir+'/', push_location, ssh=usessh,
+                 excludes=final_exclusions)
 
-    set_push_data(tree, push_location)
+        set_push_data(tree, push_location)
+    finally:
+        tree.unlock()
 
 
 def short_committer(committer):
@@ -296,8 +352,11 @@ def apache_ls(t):
         ' <a href="'
     t = t.clone()
     t._remote_path = lambda x: t.base
-    lines = t.get('')
-    expr = re.compile('<a[^>]*href="([^>]*)"[^>]*>', flags=re.I)
+    try:
+        lines = t.get('')
+    except bzrlib.errors.NoSuchFile:
+        return
+    expr = re.compile('<a[^>]*href="([^>]*)\/"[^>]*>', flags=re.I)
     for line in lines:
         match = expr.search(line)
         if match is None:
@@ -310,62 +369,39 @@ def apache_ls(t):
         yield url.rstrip('/')
 
 
-def iter_branches(t, lister=None):
-    """Iterate through all the branches under a transport"""
-    for bzrdir in iter_bzrdirs(t, lister):
-        try:
-            branch = bzrdir.open_branch()
-            if branch.bzrdir is bzrdir:
-                yield branch
-        except (NotBranchError, UnsupportedFormatError):
-            pass
+def list_branches(t):
+    def is_inside(branch):
+        return bool(branch.base.startswith(t.base))
+
+    if t.base.startswith('http://'):
+        def evaluate(bzrdir):
+            try:
+                branch = bzrdir.open_branch()
+                if is_inside(branch):
+                    return True, branch
+                else:
+                    return True, None
+            except NotBranchError:
+                return True, None
+        return [b for b in BzrDir.find_bzrdirs(t, list_current=apache_ls,
+                evaluate=evaluate) if b is not None]
+    elif not t.listable():
+        raise BzrCommandError("Can't list this type of location.")
+    return [b for b in BzrDir.find_branches(t) if is_inside(b)]
+
+
+def evaluate_branch_tree(bzrdir):
+    try:
+        tree, branch = bzrdir._get_tree_branch()
+    except NotBranchError:
+        return True, None
+    else:
+        return True, (branch, tree)
 
 
 def iter_branch_tree(t, lister=None):
-    for bzrdir in iter_bzrdirs(t, lister):
-        try:
-            wt = bzrdir.open_workingtree()
-            yield wt.branch, wt
-        except NoWorkingTree, UnsupportedFormatError:
-            try:
-                branch = bzrdir.open_branch()
-                if branch.bzrdir is bzrdir:
-                    yield branch, None
-            except (NotBranchError, UnsupportedFormatError):
-                continue
-
-
-def iter_bzrdirs(t, lister=None):
-    if lister is None:
-        def lister(t):
-            return t.list_dir('.')
-    try:
-        bzrdir = bzrdir_from_transport(t)
-        yield bzrdir
-    except (ConnectionError):
-        raise
-    except (NotBranchError, UnsupportedFormatError, TransportError,
-            PermissionDenied):
-        pass
-    try:
-        for directory in lister(t):
-            if directory == ".bzr":
-                continue
-            try:
-                subt = t.clone(directory)
-            except UnicodeDecodeError:
-                continue
-            for bzrdir in iter_bzrdirs(subt, lister):
-                yield bzrdir
-    except (NoSuchFile, PermissionDenied, TransportError):
-        pass
-
-
-def bzrdir_from_transport(t):
-    """Open a bzrdir from a transport (not a location)"""
-    format = BzrDirFormat.find_format(t)
-    BzrDir._check_supported(format, False)
-    return format.open(t)
+    return (x for x in BzrDir.find_bzrdirs(t, evaluate=evaluate_branch_tree,
+            list_current=lister) if x is not None)
 
 
 def open_from_url(location):
