@@ -126,11 +126,13 @@ class Grapher(object):
             self.common = []
 
         self.n_history = branch.revision_history()
+        self.n_revnos = branch.get_revision_id_to_revno_map()
         self.distances = node_distances(self.descendants, self.ancestors,
                                         self.root)
         if other_branch is not None:
             self.base = select_farthest(self.distances, self.common)
             self.m_history = other_branch.revision_history()
+            self.m_revnos = other_branch.get_revision_id_to_revno_map()
             new_graph = getattr(branch.repository, 'get_graph', lambda: None)()
             if new_graph is None:
                 self.new_base = None
@@ -144,6 +146,15 @@ class Grapher(object):
             self.new_base = None
             self.lcas = set()
             self.m_history = []
+            self.m_revnos = {}
+
+    @staticmethod
+    def _get_revno_str(prefix, revno_map, revision_id):
+        try:
+            revno = revno_map[revision_id]
+        except KeyError:
+            return None
+        return '%s%s' % (prefix, '.'.join(str(n) for n in revno))
 
     def dot_node(self, node, num):
         try:
@@ -155,7 +166,11 @@ class Grapher(object):
         except ValueError:
             m_rev = None
         if (n_rev, m_rev) == (None, None):
-            name = node[-5:]
+            name = self._get_revno_str('r', self.n_revnos, node)
+            if name is None:
+                name = self._get_revno_str('R', self.m_revnos, node)
+            if name is None:
+                name = node[-5:]
             cluster = None
         elif n_rev == m_rev:
             name = "rR%d" % n_rev
@@ -294,7 +309,7 @@ def write_ancestry_file(branch, filename, collapse=True, antialias=True,
             done = True
         except NoDot, e:
             raise BzrCommandError("Can't find 'dot'.  Please ensure Graphviz"\
-                " is installed correctly, or use --noantialias")
+                " is installed correctly.")
     elif ext == 'dot' and not done:
         my_file = file(filename, 'wb')
         for fragment in output:
@@ -304,7 +319,7 @@ def write_ancestry_file(branch, filename, collapse=True, antialias=True,
             invoke_dot_html(output, filename)
         except NoDot, e:
             raise BzrCommandError("Can't find 'dot'.  Please ensure Graphviz"\
-                " is installed correctly, or use --noantialias")
+                " is installed correctly.")
     elif not done:
         print "Unknown file extension: %s" % ext
 
