@@ -60,6 +60,8 @@ class Shelf(object):
     def display(self, patch=None):
         if patch is None:
             path = self.last_patch()
+            if path is None:
+                raise CommandError("No patches on shelf.")
         else:
             path = self.__path_from_user(patch)
         sys.stdout.write(open(path).read())
@@ -250,7 +252,10 @@ class Shelf(object):
         if reverse:
             args.append('-R')
         if dry_run:
-            args.append('--dry-run')
+            if sys.platform.startswith('freebsd'):
+                args.append('--check')
+            else:
+                args.append('--dry-run')
             stderr = subprocess.PIPE
         else:
             stderr = None
