@@ -1,7 +1,12 @@
-from bzrlib.tests.blackbox import ExternalBase
-from unittest import makeSuite
+import os
 import os.path
-class TestBzrTools(ExternalBase):
+from unittest import makeSuite
+
+from bzrlib import branch
+from bzrlib.tests import TestCaseWithTransport
+
+
+class TestBzrTools(TestCaseWithTransport):
     @staticmethod
     def touch(filename):
         file(filename, 'wb').write('')
@@ -91,6 +96,18 @@ class TestBzrTools(ExternalBase):
         self.assertIs(False, os.path.exists('checkout'))
         self.assertIs(True, os.path.exists('source'))
 
+    def test_zap_modified(self):
+        tree = self.make_branch_and_tree('source')
+        checkout = tree.branch.create_checkout('checkout', lightweight=True)
+        self.build_tree(['checkout/file'])
+        checkout.add('file')
+        self.run_bzr_error(('This checkout has uncommitted changes',),
+                           'zap checkout')
+        self.failUnlessExists('checkout')
+        self.run_bzr('zap checkout --force')
+        self.failIfExists('checkout')
+        self.failUnlessExists('source')
+
     def test_zap_branch(self):
         self.run_bzr('init source')
         self.run_bzr('checkout --lightweight source checkout')
@@ -115,7 +132,6 @@ class TestBzrTools(ExternalBase):
         self.assertIs(True, 'source/subsource' in lines)
         self.assertIs(True, 'checkout/subcheckout' in lines)
         self.assertIs(True, 'checkout' not in lines)
-        self.assertIs(True, 'checkout/.bzr/subcheckout' not in lines)
 
     def test_import_upstream(self):
         self.run_bzr('init source')
